# oops-repository is Copyright 2011 Canonical Ltd.
#
# Canonical Ltd ("Canonical") distributes the oops-repository source code under
# the GNU Affero General Public License, version 3 ("AGPLv3"). See the file
# LICENSE in the source tree for more information.

"""Test helpers for working with cassandra."""

import os.path

from fixtures import Fixture, TempDir
import pycassa
from pycassa.system_manager import SystemManager

from oopsrepository.cassandra import workaround_1779
from oopsrepository.schema import create


class TemporaryKeyspace(Fixture):
    """Create a temporary keyspace.

    The keyspace is named after a tempdir.
    """

    def setUp(self):
        super(TemporaryKeyspace, self).setUp()
        tempdir = self.useFixture(TempDir())
        self.keyspace = os.path.basename(tempdir.path)
        self.mgr = SystemManager()
        workaround_1779(self.mgr.create_keyspace, self.keyspace,
            replication_factor=1)
        self.addCleanup(workaround_1779, self.mgr.drop_keyspace,
            self.keyspace)


class TemporaryOOPSDB(Fixture):
    """Create a temporary usable OOPS DB.
    
    The keyspace for it is at self.keyspace.
    """

    def setUp(self):
        super(TemporaryOOPSDB, self).setUp()
        self.keyspace = self.useFixture(TemporaryKeyspace()).keyspace
        create(dict(keyspace=self.keyspace))
