# oops-repository is Copyright 2011 Canonical Ltd.
#
# Canonical Ltd ("Canonical") distributes the oops-repository source code under
# the GNU Affero General Public License, version 3 ("AGPLv3"). See the file
# LICENSE in the source tree for more information.

import json
import time
import uuid

import pycassa
from testtools import TestCase

from oopsrepository import oopses
from oopsrepository.testing.cassandra import TemporaryOOPSDB


class TestPrune(TestCase):

    def test_fresh_oops_kept(self):
        keyspace = self.useFixture(TemporaryOOPSDB()).keyspace
        config = dict(keyspace=keyspace)
        day_key = oopses.insert(config, 'key',
            json.dumps({"date":time.time(), "URL":'a bit boring'}))
        pool = pycassa.connect(keyspace)
        dayoopses_cf = pycassa.ColumnFamily(pool, 'DayOOPS')
        oopses_cf = pycassa.ColumnFamily(pool, 'OOPS')
        oopses.prune(config)
        # The oops is still readable and the day hasn't been purged.
        oopses_cf.get('key')
        dayoopses_cf.get(day_key)

    def test_old_oops_deleted(self):
        keyspace = self.useFixture(TemporaryOOPSDB()).keyspace
        config = dict(keyspace=keyspace)
        pool = pycassa.connect(keyspace)
        dayoopses_cf = pycassa.ColumnFamily(pool, 'DayOOPS')
        datestamp = time.time() - oopses.MONTH - oopses.DAY
        day_key = time.strftime('%Y%m%d', time.gmtime(datestamp))
        now_uuid = uuid.uuid1()
        dayoopses_cf.insert(day_key, {now_uuid:'key'})
        oopses_cf = pycassa.ColumnFamily(pool, 'OOPS')
        oopses_cf.insert('key',
            {"date":json.dumps(datestamp), "URL":'a bit boring'})
        oopses.prune(config)
        self.assertRaises(pycassa.NotFoundException, oopses_cf.get, 'key')
        # The day index is cleared out too.
        self.assertRaises(pycassa.NotFoundException, dayoopses_cf.get, day_key)


class TestInsert(TestCase):

    def test_insert_oops(self):
        keyspace = self.useFixture(TemporaryOOPSDB()).keyspace
        oopsid = 'booyah'
        oops = json.dumps({'duration': 13000})
        config = dict(keyspace=keyspace)
        day_key = oopses.insert(config, oopsid, oops)
        pool = pycassa.connect(keyspace)
        oopses_cf = pycassa.ColumnFamily(pool, 'OOPS')
        # The oops is retrievable
        columns = oopses_cf.get(oopsid)
        self.assertEqual('13000', columns['duration'])
        # The oops has been indexed by day
        pool = pycassa.connect(keyspace)
        dayoops_cf = pycassa.ColumnFamily(pool, 'DayOOPS')
        oops_refs = dayoops_cf.get(day_key)
        self.assertEqual([oopsid], oops_refs.values())
        ## TODO - the aggregates for the OOPS have been updated.
